/*
 * PROJECT: matereal at http://mr.digitalmuseum.jp/
 * ----------------------------------------------------------------------------
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is matereal.
 *
 * The Initial Developer of the Original Code is Jun KATO.
 * Portions created by the Initial Developer are
 * Copyright (C) 2009 Jun KATO. All Rights Reserved.
 *
 * Contributor(s): Jun KATO
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 */
package jp.digitalmuseum.mr.core;

import java.util.concurrent.ScheduledExecutorService;

/**
 * Service group.
 *
 * @author Jun KATO
 */
public class ServiceGroupImpl extends ServiceGroupAbstractImpl {
	final private static String THREAD_NAME = "Service Thread: ";
	private static long groupId;
	private String threadName;
	private long myGroupId;
	private boolean isDisposing = false;
	static {
		groupId = 0;
	}

	public ServiceGroupImpl() {
		super();
		initialize(null, DEFAULT_PERIOD);
	}

	public ServiceGroupImpl(String name) {
		super();
		initialize(name, DEFAULT_PERIOD);
	}

	public ServiceGroupImpl(long period) {
		super();
		initialize(null, period);
	}

	public ServiceGroupImpl(String name, long period) {
		super();
		initialize(name, period);
	}

	public ServiceGroupImpl(Service service) {
		super();
		registerService(service);
		initialize(null, DEFAULT_PERIOD);
	}

	public ServiceGroupImpl(Service service, long period) {
		super();
		registerService(service);
		initialize(null, period);
	}

	public ServiceGroupImpl(Service service, String name) {
		super();
		registerService(service);
		initialize(name, DEFAULT_PERIOD);
	}

	public ServiceGroupImpl(Service service, String name, long period) {
		super();
		registerService(service);
		initialize(name, period);
	}

	private void initialize(String threadName, long period) {
		myGroupId = groupId ++;
		if (threadName == null) {
			this.threadName = THREAD_NAME+myGroupId;
		} else {
			this.threadName = threadName;
		}
		Matereal.getInstance().registerServiceGroup(this);
		setInterval(period);
	}

	ScheduledExecutorService getThreadPoolExecutor() {
		return Matereal.getInstance().getThreadPoolExecutor();
	}

	final public String getName() {
		return threadName;
	}

	final synchronized public boolean isDisposing() {
		return isDisposing;
	}

	@Override
	final synchronized public void dispose() {
		isDisposing = true;
		final Matereal matereal = Matereal.getInstance();
		if (!matereal.isDisposing()) {
			Matereal.getInstance().unregisterServiceGroup(this);
		}
		super.dispose();
		isDisposing = false;
	}

	@Override
	public String toString() {
		return getName();
	}
}
