package sample;

import java.awt.BasicStroke;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;

import jp.digitalmuseum.mr.Matereal;
import jp.digitalmuseum.mr.entity.NetTansor;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.mr.task.Follow;
import jp.digitalmuseum.mr.task.Move;
import jp.digitalmuseum.mr.task.Task;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;


/**
 * Show a frame with a view of the world coordinate.
 *
 * @author Jun KATO
 */
public class Step4_ClickAndFollow {
	private Task workingTask;
	private ScreenPosition clickedPosition;

	public static void main(String[] args) {
		new Step4_ClickAndFollow();
	}

	public Step4_ClickAndFollow() {

		// Initialize a robot.
		final NetTansor robot = new NetTansor("NetTansor Web", "http://192.168.1.103:8081");
		final NetTansor robot2 = new NetTansor("NetTansor", "http://192.168.1.104:8081");

		// Run a camera.
		final Camera camera = new Camera("filter:1");
		camera.start();

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.setImageProvider(camera);
		detector.put(new NapMarker("markers/4x4_112.patt", 4.5), robot);
		detector.put(new NapMarker("markers/4x4_78.patt", 4.5), robot2);
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(
				new TypicalMDCPane(detector));

		// Initialize a main panel.
		final ImageProviderPanel panel = new ImageProviderPanel(camera) {
			private static final long serialVersionUID = 1L;
			@Override public void paintComponent(Graphics g) {
				super.paintComponent(g);
				final Graphics2D g2 = (Graphics2D) g;

				// Paint detection results.
				g2.setStroke(new BasicStroke(4));
				g2.setColor(Color.orange);
				detector.paint(g2);

				// Paint the destination.
				if (clickedPosition != null) {
					g2.setColor(Color.green);
					g2.fillOval(
							clickedPosition.getX()-5,
							clickedPosition.getY()-5,
							10, 10);
				}
			}
			@Override public void dispose() {
				configFrame.dispose();
				super.dispose();
				Matereal.getInstance().dispose();
			}
		};

		// Add a mouse listener.
		panel.addMouseListener(new MouseAdapter() {
			final private ScreenPosition clickedPosition =
					new ScreenPosition();
			final private Position destination =
					new Position();
			@Override public void mouseClicked(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1) {

					// Get a clicked position.
					clickedPosition.set(e.getX(), e.getY());
					panel.getScreenToImageOut(clickedPosition, clickedPosition);
					camera.screenToRealOut(clickedPosition, destination);

					// Assign a task moving to the position.
					if (workingTask != null && workingTask.isStarted()) {
						workingTask.stop();
					}
					workingTask = new Move(destination);
					if (workingTask.assign(robot)) {
						workingTask.start();
					}
				}
			}
		});

		final Follow follow = new Follow(robot);
		if (follow.assign(robot2)) {
			follow.start();
		}

		// Show a world.
		final JFrame frame = new DisposeOnCloseFrame(panel);
		frame.setSize(710, 660);
	}
}
