package sample.test;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JFrame;

import sample.application.VisualizedClickAndRun;

import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.core.entity.Robot;
import jp.digitalmuseum.mr.core.message.Event;
import jp.digitalmuseum.mr.core.message.EventListener;
import jp.digitalmuseum.mr.core.message.ImageUpdateEvent;
import jp.digitalmuseum.mr.core.service.CoordProvider;
import jp.digitalmuseum.mr.core.task.Task;
import jp.digitalmuseum.mr.core.task.VectorFieldTask;
import jp.digitalmuseum.mr.entity.MindstormsNXT;
import jp.digitalmuseum.mr.gui.*;
import jp.digitalmuseum.mr.resource.WheelsController;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.task.Move;
import jp.digitalmuseum.mr.task.Stop;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;

/**
 * Click and run!
 *
 * @author Jun KATO
 */
public class VisualizedClickAndRun {
	private VectorFieldTask moveTo;
	final private ScreenPosition goalPosition = new ScreenPosition();

	public static void main(String[] args) {
		new VisualizedClickAndRun();
	}

	public VisualizedClickAndRun() {

		// Initialize a robot.
		final Robot robot = new MindstormsNXT("Mindstorms NXT", "btspp://00165306523e");

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.setImageProvider(camera);
		detector.put(new NapMarker("markers/4x4_743.patt", 12), robot);
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(
				new TypicalMDCPane(detector));

		// Make and show a window for showing captured image.
		final DrawableFrame frame = new DrawableFrame() {
			private static final long serialVersionUID = 1L;
			private final Stroke boldStroke = new BasicStroke(3);
			private final AlphaComposite halfTransparent =
					AlphaComposite.getInstance(AlphaComposite.SRC_OVER, .3f);

			@Override public void dispose() {
				super.dispose();
				configFrame.dispose();
				Matereal.getInstance().dispose();
			}

			@Override
			public void paint2D(Graphics2D g) {
				camera.drawImage(g);

				final Composite comp = g.getComposite();
				final Stroke s = g.getStroke();
				g.setComposite(halfTransparent);
				g.setStroke(boldStroke);

				//
				g.setColor(Color.cyan);
				camera.getRectangle().draw(g);

				// Draw vectors.
				g.setColor(Color.green);
				GUI.drawVectors(camera, moveTo, g);
				g.setColor(Color.white);
				g.setComposite(comp);

				final Task task = robot.getAssignedTask(WheelsController.class);
				if (task != null) {
					g.drawString("Status: "+task, 10, 30);

					// Draw the destination.
					if (task instanceof Move) {
						g.drawLine(
								goalPosition.getX()-5,
								goalPosition.getY()-5,
								goalPosition.getX()+5,
								goalPosition.getY()+5);
						g.drawLine(
								goalPosition.getX()-5,
								goalPosition.getY()+5,
								goalPosition.getX()+5,
								goalPosition.getY()-5);
					}
				}
				else {
					g.drawString("Status: Goal not specified", 10, 30);
				}
				g.setStroke(s);
				g.drawLine(10, 35, 190, 35);
			}
		};
		frame.setFrameSize(camera.getWidth(), camera.getHeight());

		// Go to the clicked location when clicked.
		frame.getPanel().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseReleased(MouseEvent e) {
				goalPosition.set(e.getX(), e.getY());
				final Task task = robot.getAssignedTask(WheelsController.class);
				if (task != null) {
					task.stop();
				}
				moveTo = new Move(camera.screenToReal(goalPosition));
				moveTo.setDefaultTransition(new Stop());
				if (moveTo.assign(robot)) {
					moveTo.start();
				}
			}

		});

		// Switch the view.
		frame.getPanel().addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent arg0) {

			}
		});

		// Repaint the window periodically.
		camera.addEventListener(new EventListener() {
			public void eventOccurred(Event e) {
				if (e instanceof ImageUpdateEvent) {
					frame.repaint();
				}
			}
		});
	}

	private static class GUI {
		private static final double LENGTH = 2;
		private static final int nx = 30;
		private static final Position p = new Position();
		private static final Position p2 = new Position();
		private static final ScreenPosition sp = new ScreenPosition();
		private static final ScreenPosition sp2 = new ScreenPosition();

		public static void drawVectors(CoordProvider coordProvider, VectorFieldTask vectorTask, Graphics2D g) {
			final double d = coordProvider.getRealWidth()/(nx);
			final int ny = (int) (coordProvider.getRealHeight()/d)+1;
			for (int i = 0; i <= nx; i ++) {
				p.setX(d*i);
				for (int j = 0; j <= ny; j ++) {
					p.setY(d*j);
					if (vectorTask != null) {
						vectorTask.getVectorOut(p, p2);
						final double l = p2.getNorm()/LENGTH;
						p2.set(p.getX()+p2.getX()/l, p.getY()+p2.getY()/l);
					} else {
						p2.set(p);
					}
					coordProvider.realToScreenOut(p, sp);
					coordProvider.realToScreenOut(p2, sp2);
					g.drawLine(sp.getX(), sp.getY(), sp2.getX(), sp2.getY());
					if (vectorTask == null) {
						continue;
					}
					final double rot = sp.getRelativeDirection(sp2);
					sp.set(
							sp2.getX()+(int)(Math.cos(rot-Math.PI*5/6)*LENGTH/3),
							sp2.getY()+(int)(Math.sin(rot-Math.PI*5/6)*LENGTH/3));
					g.drawLine(sp.getX(), sp.getY(), sp2.getX(), sp2.getY());
					sp.set(
							sp2.getX()+(int)(Math.cos(rot+Math.PI*5/6)*LENGTH/3),
							sp2.getY()+(int)(Math.sin(rot+Math.PI*5/6)*LENGTH/3));
					g.drawLine(sp.getX(), sp.getY(), sp2.getX(), sp2.getY());
				}
			}
		}
	}
}
