package sample.test.pen;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Set;

import jp.digitalmuseum.mr.core.entity.Resource;
import jp.digitalmuseum.mr.core.service.CoordProvider;
import jp.digitalmuseum.mr.core.service.LocationProvider;
import jp.digitalmuseum.mr.core.task.Task;
import jp.digitalmuseum.mr.core.task.LocationBasedTaskAbstractImpl;
import jp.digitalmuseum.mr.gui.utils.VectorFieldPainter;
import jp.digitalmuseum.mr.resource.Pen;
import jp.digitalmuseum.mr.task.Move;
import jp.digitalmuseum.utils.Position;
import jp.digitalmuseum.utils.ScreenPosition;

public abstract class DrawPath extends LocationBasedTaskAbstractImpl {

	private final VectorFieldPainter vectorFieldPainter = new VectorFieldPainter();
	private final Position p = new Position();
	private final ScreenPosition sp = new ScreenPosition();
	private Pen pen;
	private boolean ok;

	private BufferedImage image;
	private Graphics2D g;
	private final ScreenPosition sp_ = new ScreenPosition();

	@Override
	protected void onAssigned() {
		vectorFieldPainter.setVectorLength(1);
		pen = (Pen) getResourceMap().get(Pen.class);
	}

	@Override
	public String getName() {
		return "Draw a path / " + getClass().getSimpleName() +
				(hasDelegate() ? " ("+getDelegationChild().toString()+")" : "");
	}

	@Override
	public Set<Class<? extends Resource>> getRequirements() {
		final Set<Class<? extends Resource>> resourceTypes = super.getRequirements();
		resourceTypes.add(Pen.class);
		return resourceTypes;
	}

	protected Pen getPen() {
		return pen;
	}

	public BufferedImage getImage() {
		return image;
	}

	protected void paint(Graphics2D g2) {

		final LocationProvider locationProvider =
			getLocationProvider();
		if (locationProvider != null) {
			final CoordProvider coordProvider =
				getLocationProvider().getCoordProvider();

			// Save graphics preference.
			final Color color = g2.getColor();
			final Composite comp = g2.getComposite();
			g2.setComposite(AlphaComposite.SrcOver);

			// Draw dot at the position of the robot.
			if (image == null) {
				image = new BufferedImage(
						coordProvider.getWidth(),
						coordProvider.getHeight(),
						BufferedImage.TYPE_INT_ARGB);
				g = image.createGraphics();
			}
			getPositionOut(p);
			coordProvider.realToScreenOut(p, sp);
			g.setColor(Color.blue);
			if (sp_.isNotFound()) {
				g.drawRect(sp.getX(), sp.getY(), 1, 1);
			} else {
				g.drawLine(sp.getX(), sp.getY(), sp_.getX(), sp_.getY());
			}
			sp_.set(sp);
			g2.drawImage(image, 0, 0, null);

			// Restore graphics preference.
			g2.setColor(color);
			g2.setComposite(comp);

			// Draw tasks.
			if (hasDelegate()) {
				final Task task = getDelegationChild();
				if (task instanceof Move) {

					// Draw the vector field.
					final Move moveTo =
							(Move) task;
					vectorFieldPainter.paint(coordProvider, moveTo, g2);

					// Draw the destination.
					g2.setComposite(AlphaComposite.SrcOver);
					g2.setColor(Color.red);
					moveTo.getDestinationOut(p);
					coordProvider.realToScreenOut(p, sp);
					final int x = sp.getX(), y = sp.getY();
					g2.drawOval(x-5, y-5, 10, 10);
					g2.fillOval(x-3, y-3, 6, 6);

					// Restore graphics preference.
					g2.setColor(color);
					g2.setComposite(comp);
				}
			}
		}
	}

	protected void fireDefaultTransition() {
		ok = true;
	}

	@Override
	protected boolean checkDefaultTransition() {
		return ok;
	}
}
