package sample.test.pen.logic;

import java.awt.Graphics2D;
import java.util.Set;

import jp.digitalmuseum.mr.core.entity.Resource;
import jp.digitalmuseum.mr.core.service.CoordProvider;
import jp.digitalmuseum.mr.core.task.Task;
import jp.digitalmuseum.mr.core.task.LocationBasedTaskAbstractImpl;
import jp.digitalmuseum.mr.core.task.VectorFieldTask;
import jp.digitalmuseum.mr.gui.utils.VectorFieldPainter;
import jp.digitalmuseum.mr.resource.PenController;
import jp.digitalmuseum.mr.task.Move;
import jp.digitalmuseum.mr.task.RotateTo;
import jp.digitalmuseum.utils.Position;

public class DrawLine extends LocationBasedTaskAbstractImpl {
	private Position departure;
	private Position destination;
	private int step;
	private boolean ok = false;

	public DrawLine(Position departure, Position destination) {
		this.departure = new Position(departure);
		this.destination = new Position(destination);
		step = 0;
	}

	@Override
	public String getName() {
		return "Draw a line from "+departure+" to "+destination
		/*+(hasDelegate() ? ", "+getDelegatedTask().toString() : "")*/;
	}

	@Override
	public Set<Class<? extends Resource>> getRequirements() {
		final Set<Class<? extends Resource>> resourceTypes = super.getRequirements();
		resourceTypes.add(PenController.class);
		return resourceTypes;
	}

	@Override
	public void onAssigned() {
		super.onAssigned();
		PenController pen = (PenController) getResourceMap().get(PenController.class);
		pen.putPen(); // Make sure that the pen is down.
	}

	@Override
	public void run() {
		switch (step) {
		case 0:
			Task moveToDeparture = new Move(departure);
			moveToDeparture.setDefaultTransition(this);
			if (delegateToTask(moveToDeparture)) {
				step ++;
			}
			break;
		case 1:
			Task rotateTo = new RotateTo(getPosition().getRelativeDirection(destination));
			rotateTo.setDefaultTransition(this);
			if (delegateToTask(rotateTo)) {
				step ++;
			}
			break;
		case 2:
			Task moveToDestination = new Move(destination);
			moveToDestination.setDefaultTransition(this);
			if (delegateToTask(moveToDestination)) {
				step ++;
			}
			break;
		default:
			ok = true;
			break;
		}
	}

	public void paint(VectorFieldPainter vectorFieldPainter, Graphics2D g) {
		if (hasDelegate()) {
			final Task task = getDelegationChild();
			if (task instanceof VectorFieldTask) {
				final VectorFieldTask vectorTask =
						(VectorFieldTask) task;
				final CoordProvider coordProvider =
						vectorTask.getLocationProvider().getCoordProvider();
				vectorFieldPainter.paint(coordProvider, vectorTask, g);
			}
		}
	}

	@Override
	protected boolean checkDefaultTransition() {
		return ok;
	}
}
