package sample;

import java.awt.Color;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.util.Set;
import javax.swing.JFrame;

import jp.digitalmuseum.mr.core.Entity;
import jp.digitalmuseum.mr.core.EntityImpl;
import jp.digitalmuseum.mr.core.Matereal;
import jp.digitalmuseum.mr.gui.DisposeOnCloseFrame;
import jp.digitalmuseum.mr.gui.ImageProviderPanel;
import jp.digitalmuseum.mr.gui.utils.EntityPainter;
import jp.digitalmuseum.mr.service.MarkerDetector;
import jp.digitalmuseum.mr.service.Camera;
import jp.digitalmuseum.napkit.NapDetectionResult;
import jp.digitalmuseum.napkit.NapMarker;
import jp.digitalmuseum.napkit.gui.TypicalMDCPane;
import jp.digitalmuseum.utils.ScreenPosition;
import jp.digitalmuseum.utils.ScreenRectangle;

/**
 * Run a marker detection and show results.
 *
 * @author Jun KATO
 */
public class Step2_PaintEntity {

	public static void main(String[] args) {
		new Step2_PaintEntity();
	}

	public Step2_PaintEntity() {

		// Run a camera.
		final Camera camera = new Camera();
		camera.start();
		Matereal.getInstance().setInterval(camera.getActualInterval());

		// Run a marker detector.
		final MarkerDetector detector = new MarkerDetector();
		detector.loadCameraParameter("calib_qcam.dat");
		detector.start();

		// Show a configuration window.
		final JFrame configFrame = new DisposeOnCloseFrame(new TypicalMDCPane(detector));

		// Detect a marker.
		final NapMarker marker = new NapMarker("markers\\4x4_78.patt",45);
		final Entity dummy = new EntityImpl("test") {
			public void dispose() {}
			public Shape getShape() { return new Rectangle2D.Double(-10, -10, 20, 20); }
		};
		detector.put(marker, dummy);

		// Initialize a painter
		final EntityPainter painter = new EntityPainter();

		// Show detection results in real-time.
		final DisposeOnCloseFrame frame = new DisposeOnCloseFrame(
				new ImageProviderPanel(camera) {
					private static final long serialVersionUID = 1L;
					@Override public void paintComponent(Graphics g) {
						super.paintComponent(g);

						// Get detected results.
						final Set<NapDetectionResult> results = detector.getResults();

						// Draw detected number of squares.
						drawString(g, "detected: "+(results == null ? 0 : results.size()),
								10, getHeight()-10);

						// Draw each detected result.
						for (final NapDetectionResult result : results) {

							// Draw corners
							final ScreenRectangle corners = result.getSquare();
							corners.draw((Graphics2D) g, true);

							// Draw information for the square.
							final ScreenPosition point = result.getPosition();
							g.drawLine(point.getX(), point.getY(), point.getX()+55, point.getY()+43);
							g.drawRect(point.getX()+55, point.getY()+23, 200, 40);
							drawString(g, "Confidence: "+result.getConfidence(), point.getX()+64, point.getY()+40);
							drawString(g, "Position: "+point, point.getX()+64, point.getY()+56);
						}

						// Paint a entity
						painter.paint(g, dummy);
					}

					/**
					 * Draw a string with 1px simple black border.
					 */
					private void drawString(Graphics g, String s, int x, int y) {
						g.setColor(Color.black);
						g.drawString(s, x+1, y);
						g.drawString(s, x-1, y);
						g.drawString(s, x, y-1);
						g.drawString(s, x, y+1);
						g.setColor(Color.orange);
						g.drawString(s, x, y);
					}
				}) {
			private static final long serialVersionUID = 1L;
			@Override public void dispose() {
				configFrame.dispose();
				super.dispose();
				Matereal.getInstance().dispose();
			}
		};
		frame.setFrameSize(camera.getWidth(), camera.getHeight());
		frame.setResizable(false);
	}
}
